"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame, os
from resourcemanager import ResourceManager
from drawables import Animation
import cfg

class ResourceAnimation(Animation):
	duration = 1500
	nbrOfFrames = 15
	surfaceSize=(10, 20)
	fontBlack  = ResourceManager.get_font(20)
	fontYellow = ResourceManager.get_font(16)
	
	# used to cache once rendered resource animations
	frames = {}
	
	def __init__(self, center, resources):
		"""Initializes the ResourceAnimagtion."""
		Animation.__init__(self, center, ResourceAnimation.create_animation_frames(resources), ResourceAnimation.duration)
		
		self.rect = self.surface.get_rect()
		self.rect.center = center

	@classmethod
	def create_animation_frames(cls, resources):
		"""Returns the animation frames for the ResourceAnimation."""
		
		if not resources in cls.frames:
			frames = []
			for frame in xrange(cls.nbrOfFrames):
				black  = cls.fontBlack.render(str(resources), True, pygame.Color(0,0,0,128))
				yellow = cls.fontYellow.render(str(resources), True, pygame.Color(255,255,0,200))
				
				surface = pygame.Surface((black.get_rect().w, black.get_rect().h + cls.nbrOfFrames), cfg.SURFACEFLAGS)
				surface.fill((0,0,0,0))
				surface.blit(black,  (0, cls.nbrOfFrames - frame))				
				surface.blit(yellow, (2, cls.nbrOfFrames - frame + 2))
				frames.append(surface)
			
			cls.frames[resources] = frames
		
		return cls.frames[resources]
	
	def update(self, tick):
		self.lastframetime += tick
		self.frame_switch()

class WeaponAnimation(Animation):
	"""Attack animation of a weapon."""
	
	def __init__(self, center, frames, duration, startdelay=0):
		Animation.__init__(self, center, frames, duration)
		self.waittime = startdelay
		self.started = False
	
	def update(self, tick):
		self.lastframetime += tick
		
		# test, if the animation was started
		if not self.started:
			if self.lastframetime < self.waittime:
				return
			
			self.surface.fill((0,0,0,0))
			self.surface.blit(self.frames[self.frame], (0,0))
			
			self.lastframetime -= self.waittime
			self.started = True
		
		self.frame_switch()

class Flower(Animation):
	frames = []
	
	def __init__(self, center):
		if not Flower.frames:
			Flower.frames = Animation.get_animation_images(ResourceManager.get_image("menu" + os.sep + "flower.png"), 39)
		
		Animation.__init__(self, center, Flower.frames, 2000, loop=True)
	
	def update(self, tick):
		self.lastframetime += tick
		self.frame_switch()

class LawnMower(Animation):
	bg = ResourceManager.get_image("lawnmower_bg.png")
	fg = ResourceManager.get_image("lawnmower_fg.png")
	left = ResourceManager.get_image("lawnmower_l.png")
	right = ResourceManager.get_image("lawnmower_r.png")
	
	def __init__(self):
		Animation.__init__(self, (82, 15), [LawnMower.bg, LawnMower.fg, LawnMower.right, LawnMower.left])
		self.nextWavePercentage = 0
		self.wave = 0
	
	@property
	def dir(self):
		return (self.wave % 2) + 2
	
	def update(self, nextWavePercentage, wave):
		"""@param wave has to be integer."""
		self.surface.fill((0,0,0,0))
		self.surface.blit(self.frames[0], (0,0))
		
		if self.wave != wave:
			self.wave = wave
		
		if self.dir - 2:
			dir = nextWavePercentage
		else:
			dir = 100 - nextWavePercentage
		
		x = 10 + dir
		self.surface.blit(self.frames[self.dir], (x, 1))
		self.surface.blit(self.frames[1], (0,0))