"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame, os

#import pp
import cfg
from events import add_build_tower
from highscore import HighScore

from gesturedetector import GestureDetector
from tile import Tile
from level import Level, LevelFactory
from enemycontroller import EnemyController
from resourcemanager import ResourceManager
from vector2D import Vector2D
from tower import TowerMenu
from towercontroller import TowerController
from windowcontent import WindowContent
from drawables import Drawable
from chat import ChatBox
from radar import Radar
import events
import weapon

class Game(WindowContent):
	def __init__(self, level, menu, screen, endlessmode=False):
		
		if isinstance(level, list):
			self.campLevels = level
			self.nextLevel = 1
			self.level = LevelFactory.get_Level(self.campLevels[0])
		else:
			self.campLevels = None
			self.level = level
		
		WindowContent.__init__(self, screen = screen, bgImage=self.level.image, runner=None)
		self.endlessmode = endlessmode
		if self.endlessmode:
			# endless mode
			self.mode = 1
		elif not self.campLevels == None:
			#campaign mode
			self.mode = 3
		else:
			#skirmish mode
			self.mode = 2
		
		self.fps = []
		
		self.menu = menu
		self.wave = self.level.nbrOfWaves
		self.waveCD = 0
		self.resources = self.level.startresources
		self.left, self.right, self.up, self.down = False, False, False, False
		self.font = ResourceManager.get_font(24)
		self.saved = False
		
		self.score = 0
		self.won = None
		self.scoreBoardImage = ResourceManager.get_image("menu" + os.sep + "scoreLawn.png")
		self.scoreBoardSurface = pygame.Surface((self.scoreBoardImage.get_rect().w, self.scoreBoardImage.get_rect().h), cfg.SURFACEFLAGS)
		self.update_scoreboard_surface()
		
		# used for gesture detection
		self.checking = False
		self.rightPressed = False
		self.gesturePoints = []
		
		self.addResourceSound = ResourceManager.get_sound("res.wav")
		
		self.lifes = self.level.lifes
		self.time, self.frames = 1.0, 0
		
		self.playground = pygame.Surface(self.level.pxl_size)
		
		self.speed = 1000.0
		self.hovering_in_game = False
		
		self.gestureDetector = GestureDetector()		
		self.enemyControl = EnemyController(self, self.level, endlessmode)
		self.towerControl = TowerController(self, self.level, self.enemyControl)
		
		# Determine how many superweapons are available.
		self.superweapons_remaining = 1
		self.superweapon_timeout = 0
		self.superweapon_dirtyboots_image = ResourceManager.get_image("dirty_boots.png")
		self.superweapon_dirtyboots_sound = ResourceManager.get_sound("dirty_boots.wav")
		self.superweapon_dirtyboots_sound.set_volume(.8) # make it loud, given that the rest is lower ...
		
		# Chat / Message window is disabled by default.
		self.chat = None
		
		# Get a radar.
		self.radar = Radar(self.level, self.enemyControl)
		
		self.update_scroll_offset(0,(0,0))
		self.set_scroll_borders()
		
		self.deathImage = ResourceManager.get_image("lost.png")
		self.wonImage = ResourceManager.get_image("won.png")
		self.splash_surface = pygame.Surface(self.level.pxl_size, cfg.SURFACEFLAGS)
		
		# Display the intro image, if any.
		self.freeze = False
		self.intro_image = self.level.get_intro_image()
		if self.intro_image:
			self.activate_freeze("intro") # Stop updating, hold the clock, etc.
			# Since intro is the first thing to happen, we need to run the update_gamecontent()
			# method at least once. This checks, if this has occurred yet.
			self.intro_level_drawn = False
		
		self.animationGroup = pygame.sprite.Group()
		
		self.initialize_user_events()
		
		# determine if parallel python should be used for updates
		# once pygame Surfaces have object as its baseclass, we can try again to use pp to
		# increase performance
		self.updatefunc = self.nonpp_update
		#self.updater = pp.Server(ppservers=())
		#if self.updater.get_ncpus()  > 1:
		#	self.updatefunc = self.pp_update
		
	
	def initialize_user_events(self):
		self.userEvents = {}
		
		self.userEvents["BuildTower"] = self.towerControl.handle_build_tower_event
		self.userEvents["AddResources"] = self.add_resources
		self.userEvents["Animation"] = self.add_animation
		self.userEvents["Gesture"] = self.affect_gesture
	
	def initialize_nextLevel(self):
		self.bgimage = self.level.bgimage
		self.enemyControl.initialize_nextLevel(self.level)
		self.towerControl.initialize_nextLevel(self.level)
		self.wave = self.level.nbrOfWaves
		self.waveCD = 0
		self.resources = self.level.startresources
		
		# used for gesture detection
		self.checking = False
		self.rightPressed = False
		self.gesturePoints = []
		
		self.lifes = self.level.lifes
		self.time, self.frames = 1.0, 0
		
		self.hovering_in_game = False

		# Determine how many superweapons are available.
		self.superweapons_remaining = 1
		
		# Display the intro image, if any.
		self.freeze = False
		self.intro_image = self.level.get_intro_image()
		if self.intro_image:
			self.activate_freeze("intro") # Stop updating, hold the clock, etc.
			# Since intro is the first thing to happen, we need to run the update_gamecontent()
			# method at least once. This checks, if this has occurred yet.
			self.intro_level_drawn = False	

	def activate_super_weapon(self):
		if not self.superweapons_remaining:
			return
		
		# Just used one up!
		self.superweapons_remaining -= 1
		self.superweapon_dirtyboots_sound.play()
		
		# Timeout!
		self.superweapon_timeout = 800
		
	def update_super_weapon(self, tick):
		"""Update the superweapon-status."""
		if not self.superweapon_timeout:
			return
		
		self.superweapon_timeout = max(0, self.superweapon_timeout - tick)
		
		# If we're done showing it, kill all enemies.
		if 0 == self.superweapon_timeout:
			self.enemyControl.kill_em_all()
		
	def blit_super_weapon(self, surface):
		rect = self.superweapon_dirtyboots_image.get_rect()
		x = self.offset.x + self.screenwidth / 2 - rect.width/2
		y = self.offset.y + self.screenheight / 2 - rect.height/2
		
		surface.blit(self.superweapon_dirtyboots_image, (x,y))


	def nonpp_update(self, func, tick):
		func(tick)
	
	def pp_update(self, func, tick):
		self.updater.submit(func, (tick,), (), ("pygame",))
	
	def set_scroll_borders(self):
		self.minX = 0
		self.minY = 0
		
		info = pygame.display.Info()
		
		self.pg_width = info.current_w
		self.pg_height = info.current_h - self.menu.height
		
		self.screenwidth  = info.current_w
		self.screenheight = info.current_h
		
		Drawable.set_visible_region(bottomright = self.offset + (self.screenwidth, self.screenheight))
		
		self.maxX = self.level.pxl_width - self.pg_width
		self.maxY = self.level.pxl_height - self.pg_height
		
		self.menu.set_surface(info)
	
	def get_scrollspeed(self, tick):
		"""Returns the scrollspeed.
		
		@param tick Time since the last screen refresh.
		"""
		return self.speed * tick / 1000
	
	def tick(self):
		tick = self.clock.tick()
		self.fps.append(int(1000.0 / (tick + 1)))
		return tick
	
	def get_fps(self):
		minimum = min(self.fps)
		maximum = max(self.fps)
		
		avg = 0
		for item in self.fps:
			avg += item
		avg /= len(self.fps)
		
		return minimum, maximum, avg
	
	def _chatposition(self):
		# Draw chat window at the lower right bottom of the screen
		x = self.screenwidth - ChatBox.size.x - 10 + self.offset.x
		y = self.screenheight - self.menu.height - ChatBox.size.y - 40 + self.offset.y
		return (int(x),int(y))
		
	def _radarposition(self):
		# Draw chat window at the upper right bottom of the screen
		x = self.screenwidth - self.radar.size.x - 10 + self.offset.x
		y = 10 + self.offset.y
		return (int(x),int(y))
		
	def activate_freeze(self, title):
		self.freeze = title
		self.freeze_timeout = 200

	def waste_resources(self, resources):
		"""Tests if the resources could be spend.
		
		@param resources Resources to spend.
		
		@return True, if the resources could be spend, False otherwise.
		"""
		if self.resources < resources:
			return False
		
		self.resources -= resources
		return True
	
	def add_resources(self, event):
		self.resources += event.resources
		self.addResourceSound.play()
	
	def add_animation(self, event):
		animation = event.animation
		self.animationGroup.add(event.animation)
		
	def update_wave(self, waveStr):
		"""Changes the number of running wave.."""
		self.wave = waveStr
	
	def update_waveCooldown(self, cooldown):
		"""Changes the time of cooldown till the next wave starts."""
		self.waveCD = max(0, cooldown)
	
	def update_lifes(self, diff):
		"""Changes the number of lifes the player has left."""
		self.lifes = max(0, self.lifes + diff)
	
	def update_score(self, diff):
		"""Changes the players score."""
		self.score += diff
		self.update_scoreboard_surface()		
	
	def handle_events(self):
		keyboard_activity = False
		mouse_activity = False
		mouse_button_activity = False
		
		events = pygame.event.get()
		for event in events:
			if event.type == pygame.QUIT:
				WindowContent.quitApp = True
				return False
			
			elif event.type == pygame.VIDEORESIZE:
				size = list(event.size)
				if size[0] > self.level.pxl_width:
					size[0] = self.level.pxl_width
				if size[1] > self.level.pxl_height + self.menu.height:
					size[1] = self.level.pxl_height + self.menu.height	
				
				pygame.display.set_mode(size, pygame.RESIZABLE)
				self.set_scroll_borders()
				
			elif event.type == pygame.KEYDOWN:
				keyboard_activity = True
				if   event.key == pygame.K_ESCAPE:  return False
				elif event.key == pygame.K_DOWN:  self.down = True
				elif event.key == pygame.K_UP:    self.up   = True
				elif event.key == pygame.K_LEFT:  self.left = True
				elif event.key == pygame.K_RIGHT: self.right = True
				elif event.key == pygame.K_q:     return False
				elif event.key == pygame.K_m:
					if self.chat: self.chat = None
					else: self.chat = ChatBox
				#elif event.key == pygame.K_s:     self.activate_super_weapon()
				elif event.key == pygame.K_t:     add_build_tower("FrogTower")
				elif event.key == pygame.K_f:     add_build_tower("FlyFlapTower")
				elif event.key == pygame.K_i:     add_build_tower("InsecticideTower")
				#elif event.key == pygame.K_k:     self.enemyControl.kill_em_all()
				#elif event.key == pygame.K_w:     self.enemyControl.next_wave_plz()
			elif event.type == pygame.KEYUP:
				keyboard_activity = True
				if   event.key == pygame.K_DOWN:  self.down  = False
				elif event.key == pygame.K_UP:    self.up    = False
				elif event.key == pygame.K_LEFT:  self.left  = False
				elif event.key == pygame.K_RIGHT: self.right = False
				
			elif event.type == pygame.MOUSEBUTTONDOWN:
				mouse_activity = mouse_button_activity = True
				if event.pos[1] >= self.pg_height:
					# hit the menu
					position = Vector2D(event.pos[0], event.pos[1] - self.pg_height)
					self.menu.handle_mouse_event(pygame.MOUSEBUTTONDOWN, position, event.button)
				else:
					if not self.towerControl.place_tower(event):
						if event.button == 3:
							self.checking = True
							self.rightPressed = True
							pos = event.pos
							self.gesturePoints.append((self.offset[0] + pos[0], self.offset[1] + pos[1]))
				
			elif event.type == pygame.MOUSEBUTTONUP:
				mouse_activity = mouse_button_activity = True
				
				if event.button == 3:
					self.rightPressed = False
				
				if event.pos[1] >= self.pg_height:
					# hit the menu
					position = Vector2D(event.pos[0], event.pos[1] - self.pg_height + self.menu.overlap)
					self.menu.handle_mouse_event(pygame.MOUSEBUTTONUP, position, event.button)
				
			elif event.type == pygame.MOUSEMOTION:
				mouse_activity = True
				# pos: tuple of x and y coordinates
				# rel: tuple of relative x and relative y change from previous position
				# buttons: tuple of three values (left,middle,right). 0-not pressed 1-pressed
				
				# adds the current position to the gesture, if it should be checked.
				if self.checking:
					#pos = pygame.mouse.get_pos()
					pos = event.pos
					self.gesturePoints.append((self.offset[0] + pos[0], self.offset[1] + pos[1]))
					continue
				
				# Deactivate hovering
				if self.hovering_in_game:
					self.hovering_in_game = False
					self.towerControl.handle_mouse_hovering(None)
					self.enemyControl.handle_mouse_hovering(None)
					
				# Possibly hovering over a tower -> need to show its range.
				if event.pos[1] < self.pg_height:
					self.hovering_in_game = [10, event.pos]
					self.towerControl.handle_mouse_hovering(event.pos)
					self.enemyControl.handle_mouse_hovering(event.pos)
					
					# Hovering the chat window? If so, the rest is not relevant.
					if self.chat:
						chatp = self._chatposition()
						chatp = (chatp[0] - self.offset.x, chatp[1] - self.offset.y)
						chatRect = pygame.Rect(chatp[0],chatp[1],chatp[0] + ChatBox.size.x, chatp[1] + ChatBox.size.y)
						hovering_chat = chatRect.collidepoint(event.pos)
						self.chat.hovering_mouse(hovering_chat)
				
			elif event.type == pygame.USEREVENT:
				self.userEvents[event.name](event)
			
		# If no mouse event has been noticed (and we care about hovering), then the mouse is hovering now.
		if self.hovering_in_game and not mouse_activity:
			
			# Don't start hovering right away, rather wait some cycles.
			if 0 < self.hovering_in_game[0]:
				self.hovering_in_game[0] -= 1
				
			# So we are hovering -> notify the tower and save that its been taken care of.
			if 0 == self.hovering_in_game[0]:
				self.towerControl.handle_mouse_hovering(self.hovering_in_game[1])
				self.hovering_in_game[0] = -1
				
		# Deactivate any freeze if it cancelled by user input.
		if self.freeze and (keyboard_activity or mouse_button_activity):
			if 0 >= self.freeze_timeout:
				if self.freeze in ["intro"]:
					if self.freeze == "intro":
						self.intro_image = None
					self.freeze = False
			
		return True
	
	def check_for_gesture(self):
		"""Checks, if a gesture was found.
		
		@return returns the detected gesture.
		"""
		if self.checking and not self.rightPressed:
			self.checking = False
			gesture = self.gestureDetector.compare_with_gestures(self.gesturePoints)
			self.gesturePoints = []
			
			if not gesture:
				return
			
			events.add_detected_gesture(gesture)
	
	def draw_gesture(self):
		if len(self.gesturePoints) < 2:
			return
		
		pygame.draw.aalines(self.playground, (0,0,200,200), False, self.gesturePoints, 3)
	
	def affect_gesture(self, event):
		gesture = event.gesture
		
		# circle
		if gesture["id"] == 0:
			pass
		# bolt
		elif gesture["id"] == 1:
			pass
		# heart
		elif gesture["id"] == 2:
			pass
		#wave
		elif gesture["id"] == 3:
			pass
		# line topdown
		elif gesture["id"] == 4:
			self.towerControl.sell_tower(gesture["startPoint"])
		#line downtop
		elif gesture["id"] == 5:
			self.towerControl.update_tower(gesture["startPoint"])
	
	def update_gamecontent(self, tick):
		"""This function is used to calculate the complete map and all changes.
		
		@param tick Time since the last screen refresh.
		"""
		if self.freeze:
			
			# First, update freeze timeout.
			self.freeze_timeout -= tick
			
			if self.won != None:			
				self.shouldRun = self.update_end_splash(tick)
				return 
			if self.freeze == "intro":
				self.update_intro_splash(tick)
				
				# Only update game content once.
				if self.intro_level_drawn:
					return
				
				self.intro_level_drawn = True

		# gesture detection
		self.check_for_gesture()
		
		self.menu.set_credits(self.resources)
		self.menu.set_lifes(self.lifes)
		self.menu.set_wave(self.wave, self.waveCD)
		
		self.updatefunc(self.towerControl.update, tick)
		self.updatefunc(self.menu.update, tick)
		self.updatefunc(self.enemyControl.update, tick)
		self.updatefunc(self.animationGroup.update, tick)
		
		# we need to keep the chat-box up-to-date, even if is currently not shown
		self.updatefunc(ChatBox.update, tick)
		
		# Update the radar.
		self.updatefunc(self.radar.update, tick)
		
		# Update the superweapon
		self.update_super_weapon(tick)
		
		#self.updater.wait()
		
		# filters all dead weapon animations from the weaponAnimationGroup
		self.animationGroup.remove(filter(lambda deadAnim: deadAnim.finished, self.animationGroup))
		
		# send signal to the controllers, that they should draw their updated images
		self.enemyControl.draw()
		self.towerControl.draw()
		self.menu.draw()
		if self.chat or ChatBox.wants_to_be_drawn():
			ChatBox.draw()
		if self.radar:
			self.radar.draw()
		
		# Only update stuff, if the player has not lost yet.
		if self.lifes < 1:
			self.activate_freeze("game_over")
			self.won = False
			self.show_splash_screen = 2000
		elif self.enemyControl.noMoreEnemies and not self.won:
			if not self.campLevels==None and self.nextLevel<len(self.campLevels):
				self.level=LevelFactory.get_Level(self.campLevels[self.nextLevel])
				self.nextLevel +=1
				self.initialize_nextLevel()
			else:
				self.activate_freeze("game_over")
				self.won = True
				self.show_splash_screen = 2500

	def update_scoreboard_surface(self):
		self.scoreBoardSurface.fill((0,0,0,0))
		self.scoreBoardSurface.blit(self.scoreBoardImage, (0,0))
		self.scoreBoardSurface.blit(self.font.render("%s" % self.score, True, pygame.Color(255,0,0,200)), (50,33))
	
	def draw(self):
		# cleans the playground
		self.playground.blit(self.level.image, (0,0))
		
		# blits the layers of all controllers to the playground
		self.playground.blit(self.enemyControl.image, (0,0))
		self.playground.blit(self.towerControl.image, (0,0))
		
		#Show Score
		x = int(self.offset.x + 5)
		y = int(self.offset.y + 5)
		self.playground.blit(self.scoreBoardSurface, (x,y))
		
		# Draw superweapon
		if self.superweapon_timeout:
			self.blit_super_weapon(self.playground)
		
		# Chat box
		if self.chat or ChatBox.wants_to_be_drawn():
			self.playground.blit(ChatBox.image, self._chatposition())
			
		# Radar
		if self.radar:
			self.playground.blit(self.radar.image, self._radarposition())
			
		# draws all weapon animations
		self.animationGroup.draw(self.playground)
		
		# draw gesture
		if self.checking:
			self.draw_gesture()
		
		# Message splash screens
		if self.freeze:
			self.playground.blit(self.splash_surface, (0,0))
		
		scrollarea = (self.offset.x, self.offset.y, self.offset.x + self.pg_width, self.offset.y + self.pg_height)
		self.screen.blit(self.playground, (0, 0), scrollarea)	
		self.screen.blit(self.menu.image, self.menu.topleft)

		if None != self.won and not self.shouldRun:
			if self.won or self.endlessmode:
				if not self.saved:
					self.saved = True
					msg = []
					msg.append("Your Score: ")
					msg.append("Killed critters: "+ str(self.score))
					if self.lifes >0:
						msg.append("Life bonus: "+ str(self.lifes))
					if self.resources >0:
						msg.append("Resource bonus: "+ str(self.resources))
					msg.append("Total: "+ str(self.score+self.lifes+self.resources))
					import scorebox
					answer = scorebox.showScore(self.screen, msg)
					HighScore().write_highscores_to_database(answer, self.score+self.lifes+self.resources, self.mode)

	def update_end_splash(self, tick):
		"""Updates the you-are-dead splashscreen.
		
		@return True, if the animation continues, False if the game controller is finished.
		"""
		
		self.splash_surface.fill((0,0,0,0))
		
		if self.won:
			image = self.wonImage
		else:
			image = self.deathImage
		
		# count down
		self.show_splash_screen -= tick
		
		# Grey out the rest of the image.
		self.splash_surface.fill((50,50,50,200))

		# Draw the victory / defeat image.
		x = int(self.offset.x + self.screenwidth  / 2 - image.get_rect().width / 2)
		y = int(self.offset.y + self.screenheight / 2 - image.get_rect().height / 2)
		self.splash_surface.blit(image, (x,y))
		
		return 0 < self.show_splash_screen
	
	def update_intro_splash(self, tick):
		"""Updates the intro splashscreen."""
		
		# Grey out the rest of the image.
		self.splash_surface.fill((50,50,50,200))

		x = int(self.offset.x + self.screenwidth  / 2 - self.intro_image.get_rect().width / 2)
		y = int(self.offset.y + self.screenheight / 2 - self.intro_image.get_rect().height / 2)
		self.splash_surface.blit(self.intro_image, (x,y))
		
		font = ResourceManager.get_font(20)
		text_image = font.render("Press any key or button to start.", True, (255,255,255))
		x = int(self.offset.x + self.screenwidth  / 2 - text_image.get_rect().width / 2)
		y += self.intro_image.get_rect().height + 20
		self.splash_surface.blit(text_image, (x,y))
	
	def update_scroll_offset(self, tick, offset=None):
		if offset:
			self.offset = Vector2D(offset)
		elif not (self.left or self.right or self.up or self.down):
			return
		
		speed = self.get_scrollspeed(tick)
		
		if self.right:
			self.offset.x += speed
			if self.offset.x > self.maxX:
				self.offset.x = self.maxX
				self.left = False
		
		if self.left:
			self.offset.x -= speed
			if self.offset.x < self.minX:
				self.offset.x = self.minX
				self.right = False
		
		if self.down:
			self.offset.y += speed
			if self.offset.y > self.maxY:
				self.offset.y = self.maxY
				self.up = False
		
		if self.up:
			self.offset.y -= speed
			if self.offset.y < self.minY:
				self.offset.y = self.minY
				self.down = False
		
		# sets the topleft of all drawable objects
		Drawable.set_visible_region(self.offset)
		
		self.towerControl.set_offset(self.offset)
		self.enemyControl.set_offset(self.offset)

	def execution_loop(self, tick):
		self.update_scroll_offset(tick)
		self.update_gamecontent(tick)
		self.draw()

def run():
	"""Initializes the pygame sub system."""
	pygame.init()
	window = pygame.display.set_mode((800, 600), pygame.DOUBLEBUF | pygame.RESIZABLE)
	pygame.display.set_caption("@todo implement me!!!")
	
	screen = pygame.display.get_surface()		
	
	#endlessmode = True
	endlessmode = False
	g = Game(["level_05"], TowerMenu(screen), screen, endlessmode)
	g.run()
	
	print "FPS (min,max,avg): %s" % str(g.get_fps())
	
if __name__ == "__main__":
	run()
	#raw_input()
	#import cProfile
	#cProfile.run("run()", filename = "prof.cprof")
	
	#import pstats
	#stats = pstats.Stats("prof.cprof")
	#stats.strip_dirs().sort_stats("time").print_stats(30)
