"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame, os, datetime

from resourcemanager import ResourceManager
from menuitems import MenuItem
from animations import Flower

from windowcontent import WindowContent
class Menu(WindowContent):
	def __init__(self, screen, runner, parent=None):
		image = "menu" + os.sep
		if 7 <= datetime.datetime.now().hour < 19:
			image += "background_day.png"
		else:
			image += "background_night.png"
		
		WindowContent.__init__(self, screen, ResourceManager.get_image(image), runner, parent)
		self.menuItems = []
		self.menuzone = (200, 600)
		self.itemSize = (512, 70)
		self.initiate_menu_items()
		
		self.animations = pygame.sprite.Group()
		flower = Flower((600, 700))
		#self.animations.add(flower)
	
	def calculate_positions(self, numberOfItems):
		res = []
		menuheight = self.menuzone[1] - self.menuzone[0]
		padding = (menuheight - numberOfItems * self.itemSize[1])/ (2 * numberOfItems)
		
		posX = (self.itemSize[0] / 2)
		posY = self.menuzone[0]
		
		for i in range(numberOfItems):
			posY += padding
			res.append((posX, posY))
			posY += self.itemSize[1]
			
			if posY > self.menuzone[1]:
				print "Warning: Too many items for menu %s" % (self.__class__.__name__,)
			posY += padding
		
		return res
	
	def add_menu_item(self, item):
		self.menuItems.append(item)
		positions = self.calculate_positions(len(self.menuItems))
		
		for idx in range(len(self.menuItems)):
			self.menuItems[idx].set_position(positions[idx], False)
		
		self.items = pygame.sprite.RenderPlain(self.menuItems)
		
		self.currentItem = -1
		self.activate_next()
	
	def initiate_menu_items(self):
		raise NotImplementedError
	
	def activate_next(self, recursion = 0):
		if self.currentItem >= 0:
			self.menuItems[self.currentItem].deactivate()
		
		self.currentItem += 1
		if self.currentItem == len(self.menuItems):
			self.currentItem = 0
		
		result = self.menuItems[self.currentItem].activate()
		if not result:
			recursion += 1
			if recursion <= len(self.menuItems):
				result = self.activate_next(recursion)
		
		return result
	
	def activate_last(self, recursion = 0):
		if self.currentItem >= 0:
			self.menuItems[self.currentItem].deactivate()
		
		self.currentItem -= 1
		if self.currentItem < 0:
			self.currentItem = len(self.menuItems) - 1
		
		recursion += 1
		if not self.menuItems[self.currentItem].activate() and recursion < len(self.menuItems):
			self.activate_last(recursion)
	
	def handle_events(self):
		events = pygame.event.get()
		
		for event in events:
			if event.type == pygame.VIDEORESIZE:
				size = list(event.size)
				pygame.display.set_mode(size, pygame.RESIZABLE)
				self.draw_background()
			
			elif event.type == pygame.KEYDOWN:
				if event.key == pygame.K_DOWN:
					self.activate_next()
				elif event.key == pygame.K_UP:
					self.activate_last()
				elif event.key == pygame.K_ESCAPE:
					self.back_to_parent()
				elif self.currentItem >= 0:
					currentItem = self.menuItems[self.currentItem]
					if currentItem.active:
						currentItem.action(event.key)
						self.draw_background()
			elif event.type == pygame.QUIT:
				WindowContent.quitApp = True
				
		return True

	def draw(self):
		self.draw_background()

	def execution_loop(self, tick):
		self.handle_events()
		self.items.clear(self.screen, self.bgImage)
		self.animations.update(tick)
		self.items.update(tick)
		self.draw()
		self.animations.clear(self.screen, self.bgImage)
		self.animations.draw(self.screen)
		self.items.draw(self.screen)
