"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame, os

import cfg
from resourcemanager import ResourceManager
from vector2D import Vector2D

from drawables import Drawable
class Tile(Drawable):
	"""Represents a tile of the map."""
	size = (32, 32)
	vecsize = Vector2D(size)
	
	def __init__(self, color, image=None, movable=False, buildable=False, lair=False, target=False):
		Drawable.__init__(self)
		
		self.color = color
		self.image = image
		
		if not self.image:
			self.image = pygame.Surface(Tile.size, cfg.SURFACEFLAGS)
			self.image.fill(self.color)
		
		self.rect = self.image.get_rect()
		
		self.movable   = movable
		self.buildable = buildable
		self.lair      = lair
		self.target    = target

class _TileFactory(object):
	def __init__(self):
		self.mapping = {
			".": {"color":(  0,   0,   0, 0)},
			"X": {"color":(  0,   0, 255, 255), "image": ResourceManager.get_image("tiles" + os.sep + "movable_only.png"), "movable":   True},
			
			"B": {"color":(  0, 0,   0, 0), "image": ResourceManager.get_image("tiles" + os.sep + "bc.png"), "buildable": True},
			
			"T": {"color":(255,   0, 255, 255), "target":    True},
			"0": {"color":(  0, 255, 255, 255), "lair":      True}
		}
	
	def set_skin(self, skin):
		pass
	
	def get_tile(self, key):
		if not key in self.mapping:
			key = "."
		
		return Tile(**self.mapping[key])

TileFactory = _TileFactory()